# -*- coding: utf-8 -*-

"""
MEDIA:
    Cuarto Poder, Chiapas

USAGE:
    ## Get all the news from a specific date. ##
    ---------------------------------------------------------------------------------------------
    $ cd cuartoPoder/
    $ scrapy crawl noticias --nolog -s filename=2018-08-30.json -a year=2018 -a month=8 -a day=30
"""

import scrapy, re
from cuartoPoder.items import NoticiasItem
from datetime import datetime, timedelta, tzinfo


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)


class UTC(tzinfo):
    """
    Class for Time Zone
    """
    def utcoffset(self, dt):
        ## Time zone for Chiapas: UTC-6 ##
        return timedelta(hours=-6)

    def tzname(self, dt):
        ## Time zone name ##
        return 'UTC-6'



class ImportantData(scrapy.Item):
    """
    Useful data for the flow of the implementation
    """
    CONTINUE = scrapy.Field()
    page     = scrapy.Field()



class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"


    def start_requests(self):
        self.tz    = UTC()
        self.year  = getattr(self, "year", None)
        self.month = getattr(self, "month", None)
        self.day   = getattr(self, "day", None)

        self.baseURL    = "http://www.cuartopoder.mx"
        first_URL       = self.baseURL + "/archivo/portada/listado/{1}-{2}-{0}/{1}-{2}-{0}/".format(self.year, self.month.zfill(2), self.day.zfill(2))
        self.second_URL = self.baseURL + "/XStatic/cuartopoder/template/cargaBloque.aspx?strControl=ctrlArchivoResultadosPaginadoListado&"

        flow_info             = ImportantData()
        flow_info['CONTINUE'] = False
        flow_info['page']     = 2
        
        request              = scrapy.Request(url=first_URL, callback=self.parse)
        request.meta['item'] = flow_info
        
        yield request



    def parse(self, response):
        flow_info = response.meta['item']
        CONTINUE  = flow_info['CONTINUE']

        for link in response.css('ul.news-list').xpath('./li/h5/a/@href').extract():
            CONTINUE = True
            news_link = self.baseURL + link
            
            yield scrapy.Request(url=news_link, callback=self.parse_item)

        if CONTINUE:
            page     = flow_info['page']
            page_URL = self.second_URL + "p={3}&eids=&fd={1}-{2}-{0}&fh={1}-{2}-{0}&id=portada".format(self.year, self.month.zfill(2), self.day.zfill(2), str(page))        
            
            flow_info['CONTINUE'] = False
            flow_info['page']     += 1
            
            request              = scrapy.Request(url=page_URL, callback=self.parse)
            request.meta['item'] = flow_info

            yield request



    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        topic = response.css('div.big-title').xpath('./h2/a/span').extract_first()
        if topic is not None:
            topic = remove_tags(topic)

        for p in response.css('div.post-content').css('p').extract():
            p = remove_tags(p)
            text += p + "\n"

        ## News item info ##
        item['date']  = datetime(int(self.year), int(self.month), int(self.day), tzinfo=self.tz).isoformat("T")
        item['title'] = remove_tags(response.css('div.post-title').css('h1').extract_first())
        item['topic'] = topic
        item['text']  = text.strip()
        item['url']   = response.url

        yield item
